@@ -24,6 +24,7 @@
 #define _GNU_SOURCE
 #include <getopt.h>
 
+int input_channels, output_channels, spdif_channels;
 ice1712_eeprom_t card_eeprom;
 snd_ctl_t *ctl;
 
@@ -93,6 +94,20 @@ GtkWidget *hw_spdif_output_notebook;
 GtkWidget *hw_spdif_input_coaxial_radio;
 GtkWidget *hw_spdif_input_optical_radio;
 
+GtkWidget *input_interface_internal;
+GtkWidget *input_interface_front_input;
+GtkWidget *input_interface_rear_input;
+GtkWidget *input_interface_wavetable;
+
+GtkWidget *hw_breakbox_led_on_radio;
+GtkWidget *hw_breakbox_led_off_radio;
+
+GtkWidget *hw_phono_input_on_radio;
+GtkWidget *hw_phono_input_off_radio;
+
+GtkWidget *hw_spdif_switch_on_radio;
+GtkWidget *hw_spdif_switch_off_radio;
+
 GtkObject *av_dac_volume_adj[10];
 GtkObject *av_adc_volume_adj[10];
 GtkObject *av_ipga_volume_adj[10];
@@ -118,6 +133,18 @@ static void create_mixer_frame(GtkWidget *box, int stream)
 
 	if (stream <= 10) {
 		sprintf(str, "PCM Out %i", stream);
+	} else if ((card_eeprom.subvendor == ICE1712_SUBDEVICE_DMX6FIRE) && (stream == 11)) {
+		sprintf(str, "CD In L");
+	} else if ((card_eeprom.subvendor == ICE1712_SUBDEVICE_DMX6FIRE) && (stream == 12)) {
+		sprintf(str, "CD In R");
+	} else if ((card_eeprom.subvendor == ICE1712_SUBDEVICE_DMX6FIRE) && (stream == 13)) {
+		sprintf(str, "Line In L");
+	} else if ((card_eeprom.subvendor == ICE1712_SUBDEVICE_DMX6FIRE) && (stream == 14)) {
+		sprintf(str, "Line In R");
+	} else if ((card_eeprom.subvendor == ICE1712_SUBDEVICE_DMX6FIRE) && (stream == 15)) {
+		sprintf(str, "Phono/Mic In L");
+	} else if ((card_eeprom.subvendor == ICE1712_SUBDEVICE_DMX6FIRE) && (stream == 16)) {
+		sprintf(str, "Phono/Mic In R");
 	} else if (stream <= 18) {
 		sprintf(str, "H/W In %i", stream - 10);
 	} else if (stream == 19) {
@@ -329,8 +356,11 @@ static void create_mixer(GtkWidget *main, GtkWidget *notebook, int page)
 	gtk_widget_show(hbox);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
 
-
-	for (stream = 1; stream <= 20; stream++)
+	for(stream = 1; stream <= output_channels; stream ++)
+		create_mixer_frame(hbox, stream);
+	for(stream = 11; stream <= input_channels + 10; stream ++)
+		create_mixer_frame(hbox, stream);
+	for(stream = 19; stream <= spdif_channels + 18; stream ++)
 		create_mixer_frame(hbox, stream);
 }
 
@@ -357,6 +387,16 @@ static void create_router_frame(GtkWidget *box, int stream, int pos)
 		"H/W In 8"
 	};
 
+	if (card_eeprom.subvendor == ICE1712_SUBDEVICE_DMX6FIRE)
+	{
+		table[2] = "CD In L";
+                table[3] = "CD In R";
+                table[4] = "Line In L";
+                table[5] = "Line In R";
+                table[6] = "Phono/Mic In L";
+                table[7] = "Phono/Mic In R";
+	}
+
 	if (stream <= 8) {
 		sprintf(str, "H/W Out %i (%s)", stream, stream & 1 ? "L" : "R");
 	} else if (stream == 9) {
@@ -420,7 +460,7 @@ static void create_router_frame(GtkWidget *box, int stream, int pos)
 	gtk_box_pack_start(GTK_BOX(vbox), hseparator, FALSE, TRUE, 0);
 
 
-	for(idx = 0; idx < 10; idx++) {
+	for(idx = 2 - spdif_channels; idx < input_channels + 2; idx++) {
 		radiobutton = gtk_radio_button_new_with_label(group, table[idx]);
 		router_radio[stream-1][2+idx] = radiobutton;
 		group = gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton));
@@ -462,7 +502,11 @@ static void create_router(GtkWidget *main, GtkWidget *notebook, int page)
 	gtk_container_add(GTK_CONTAINER(viewport), hbox);
 
 	pos = 0;
-	for (stream = 1; stream <= 10; stream++) {
+	for (stream = 1; stream <= output_channels; stream++) {
+		if (patchbay_stream_is_active(stream))
+			create_router_frame(hbox, stream, pos++);
+	}
+	for (stream = 8; stream <= 8 + spdif_channels; stream++) {
 		if (patchbay_stream_is_active(stream))
 			create_router_frame(hbox, stream, pos++);
 	}
@@ -1084,7 +1128,7 @@ static void create_spdif_input_select(GtkWidget *box)
 	GSList *group = NULL;
 	int hide = 1;
 
-	if(card_eeprom.subvendor == ICE1712_SUBDEVICE_DELTADIO2496)
+	if((card_eeprom.subvendor == ICE1712_SUBDEVICE_DELTADIO2496) || (card_eeprom.subvendor == ICE1712_SUBDEVICE_DMX6FIRE))
 		hide = 0;
 
 	frame = gtk_frame_new("S/PDIF Input");
@@ -1119,11 +1163,202 @@ static void create_spdif_input_select(GtkWidget *box)
 		gtk_widget_hide_all(frame);
 }
 
+static void create_spdif_on_off(GtkWidget *box)
+{
+        GtkWidget *frame;
+        GtkWidget *vbox;
+        GtkWidget *radiobutton;
+        GSList *group = NULL;
+        int hide = 1;
+
+        if( card_eeprom.subvendor == ICE1712_SUBDEVICE_DMX6FIRE)
+                hide = 0;
+
+        frame = gtk_frame_new("S/PDIF On/Off");
+        gtk_widget_show(frame);
+        gtk_box_pack_start(GTK_BOX(box), frame, FALSE, TRUE, 0);
+        gtk_container_set_border_width(GTK_CONTAINER(frame), 6);
+
+        vbox = gtk_vbox_new(FALSE, 0);
+        gtk_widget_show(vbox);
+        gtk_container_add(GTK_CONTAINER(frame), vbox);
+        gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
+
+        radiobutton = gtk_radio_button_new_with_label(group, "On");
+        hw_spdif_switch_on_radio = radiobutton;
+        group = gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton));
+        gtk_widget_show(radiobutton);
+        gtk_box_pack_start(GTK_BOX(vbox), radiobutton, FALSE, FALSE, 0);
+        gtk_signal_connect(GTK_OBJECT(radiobutton), "toggled",
+                          (GtkSignalFunc)spdif_on_off_toggled,
+                          (gpointer)"On");
+
+        radiobutton = gtk_radio_button_new_with_label(group, "Off");
+        hw_spdif_switch_off_radio = radiobutton;
+        group = gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton));
+        gtk_widget_show(radiobutton);
+        gtk_box_pack_start(GTK_BOX(vbox), radiobutton, FALSE, FALSE, 0);
+        gtk_signal_connect(GTK_OBJECT(radiobutton), "toggled",
+                          (GtkSignalFunc)spdif_on_off_toggled,
+                          (gpointer)"Off");
+
+        if(hide)
+                gtk_widget_hide_all(frame);
+}
+
+static void create_breakbox_led(GtkWidget *box)
+{
+        GtkWidget *frame;
+        GtkWidget *vbox;
+        GtkWidget *radiobutton;
+        GSList *group = NULL;
+        int hide = 1;
+
+        if( card_eeprom.subvendor == ICE1712_SUBDEVICE_DMX6FIRE)
+                hide = 0;
+
+        frame = gtk_frame_new("Breakbox LED On/Off");
+        gtk_widget_show(frame);
+        gtk_box_pack_start(GTK_BOX(box), frame, FALSE, TRUE, 0);
+        gtk_container_set_border_width(GTK_CONTAINER(frame), 6);
+
+        vbox = gtk_vbox_new(FALSE, 0);
+        gtk_widget_show(vbox);
+        gtk_container_add(GTK_CONTAINER(frame), vbox);
+        gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
+
+        radiobutton = gtk_radio_button_new_with_label(group, "On");
+        hw_breakbox_led_on_radio = radiobutton;
+        group = gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton));
+        gtk_widget_show(radiobutton);
+        gtk_box_pack_start(GTK_BOX(vbox), radiobutton, FALSE, FALSE, 0);
+        gtk_signal_connect(GTK_OBJECT(radiobutton), "toggled",
+                          (GtkSignalFunc)breakbox_led_toggled,
+                          (gpointer)"On");
+
+        radiobutton = gtk_radio_button_new_with_label(group, "Off");
+        hw_breakbox_led_off_radio = radiobutton;
+        group = gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton));
+        gtk_widget_show(radiobutton);
+        gtk_box_pack_start(GTK_BOX(vbox), radiobutton, FALSE, FALSE, 0);
+        gtk_signal_connect(GTK_OBJECT(radiobutton), "toggled",
+                          (GtkSignalFunc)breakbox_led_toggled,
+                          (gpointer)"Off");
+
+        if(hide)
+                gtk_widget_hide_all(frame);
+}
+
+static void create_phono_input(GtkWidget *box)
+{
+        GtkWidget *frame;
+        GtkWidget *vbox;
+        GtkWidget *radiobutton;
+        GSList *group = NULL;
+        int hide = 1;
+
+        if( card_eeprom.subvendor == ICE1712_SUBDEVICE_DMX6FIRE)
+                hide = 0;
+
+        frame = gtk_frame_new("Phono Input Switch");
+        gtk_widget_show(frame);
+        gtk_box_pack_start(GTK_BOX(box), frame, FALSE, TRUE, 0);
+        gtk_container_set_border_width(GTK_CONTAINER(frame), 6);
+
+        vbox = gtk_vbox_new(FALSE, 0);
+        gtk_widget_show(vbox);
+        gtk_container_add(GTK_CONTAINER(frame), vbox);
+        gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
+
+        radiobutton = gtk_radio_button_new_with_label(group, "Phono");
+        hw_phono_input_on_radio = radiobutton;
+        group = gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton));
+        gtk_widget_show(radiobutton);
+        gtk_box_pack_start(GTK_BOX(vbox), radiobutton, FALSE, FALSE, 0);
+        gtk_signal_connect(GTK_OBJECT(radiobutton), "toggled",
+                          (GtkSignalFunc)phono_input_toggled,
+                          (gpointer)"Phono");
+
+        radiobutton = gtk_radio_button_new_with_label(group, "Mic");
+        hw_phono_input_off_radio = radiobutton;
+        group = gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton));
+        gtk_widget_show(radiobutton);
+        gtk_box_pack_start(GTK_BOX(vbox), radiobutton, FALSE, FALSE, 0);
+        gtk_signal_connect(GTK_OBJECT(radiobutton), "toggled",
+                          (GtkSignalFunc)phono_input_toggled,
+                          (gpointer)"Mic");
+
+        if(hide)
+                gtk_widget_hide_all(frame);
+}
+
+static void create_input_interface(GtkWidget *box)
+{
+        GtkWidget *frame;
+        GtkWidget *vbox;
+        GtkWidget *radiobutton;
+        GSList *group = NULL;
+        int hide = 1;
+
+        if( card_eeprom.subvendor == ICE1712_SUBDEVICE_DMX6FIRE)
+                hide = 0;
+
+        frame = gtk_frame_new("Line In Selector");
+        gtk_widget_show(frame);
+        gtk_box_pack_start(GTK_BOX(box), frame, FALSE, TRUE, 0);
+        gtk_container_set_border_width(GTK_CONTAINER(frame), 6);
+
+        vbox = gtk_vbox_new(FALSE, 0);
+        gtk_widget_show(vbox);
+        gtk_container_add(GTK_CONTAINER(frame), vbox);
+        gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
+
+        radiobutton = gtk_radio_button_new_with_label(group, "Internal");
+        input_interface_internal = radiobutton;
+        group = gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton));
+        gtk_widget_show(radiobutton);
+        gtk_box_pack_start(GTK_BOX(vbox), radiobutton, FALSE, FALSE, 0);
+        gtk_signal_connect(GTK_OBJECT(radiobutton), "toggled",
+                          (GtkSignalFunc)analog_input_select_toggled,
+                          (gpointer)"Internal");
+
+        radiobutton = gtk_radio_button_new_with_label(group, "Front Input");
+        input_interface_front_input = radiobutton;
+        group = gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton));
+        gtk_widget_show(radiobutton);
+        gtk_box_pack_start(GTK_BOX(vbox), radiobutton, FALSE, FALSE, 0);
+        gtk_signal_connect(GTK_OBJECT(radiobutton), "toggled",
+                          (GtkSignalFunc)analog_input_select_toggled,
+                          (gpointer)"Front Input");
+
+        radiobutton = gtk_radio_button_new_with_label(group, "Rear Input");
+        input_interface_rear_input = radiobutton;
+        group = gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton));
+        gtk_widget_show(radiobutton);
+        gtk_box_pack_start(GTK_BOX(vbox), radiobutton, FALSE, FALSE, 0);
+        gtk_signal_connect(GTK_OBJECT(radiobutton), "toggled",
+                          (GtkSignalFunc)analog_input_select_toggled,
+                          (gpointer)"Rear Input");
+
+        radiobutton = gtk_radio_button_new_with_label(group, "Wave Table");
+        input_interface_wavetable = radiobutton;
+        group = gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton));
+        gtk_widget_show(radiobutton);
+        gtk_box_pack_start(GTK_BOX(vbox), radiobutton, FALSE, FALSE, 0);
+        gtk_signal_connect(GTK_OBJECT(radiobutton), "toggled",
+                          (GtkSignalFunc)analog_input_select_toggled,
+                          (gpointer)"Wave Table");
+
+        if(hide)
+                gtk_widget_hide_all(frame);
+}
+
 static void create_hardware(GtkWidget *main, GtkWidget *notebook, int page)
 {
 	GtkWidget *label;
 	GtkWidget *hbox;
 	GtkWidget *vbox;
+	GtkWidget *vbox2;
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_widget_show(hbox);
@@ -1140,6 +1375,11 @@ static void create_hardware(GtkWidget *main, GtkWidget *notebook, int page)
 	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, TRUE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
 
+	vbox2 = gtk_vbox_new(FALSE, 0);
+	gtk_widget_show(vbox2);
+	gtk_box_pack_end(GTK_BOX(hbox),vbox2, FALSE, FALSE, 0);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox2), 6);
+
 	create_master_clock(vbox);
 	create_rate_state(vbox);
 	create_actual_rate(vbox);
@@ -1147,7 +1387,11 @@ static void create_hardware(GtkWidget *main, GtkWidget *notebook, int page)
 
  	create_spdif_output_settings(hbox); 
 
- 	create_spdif_input_select(hbox); 
+ 	create_spdif_input_select(vbox2);
+	create_input_interface(vbox2);
+	create_breakbox_led(vbox2);
+	create_phono_input(vbox2);
+	create_spdif_on_off(vbox2);
 }
 
 static void create_about(GtkWidget *main, GtkWidget *notebook, int page)
@@ -1204,6 +1448,23 @@ static void create_analog_volume(GtkWidget *main, GtkWidget *notebook, int page)
 	GtkWidget *scrolledwindow;
 	GtkWidget *viewport;
 	int i, j;
+	static char* dmx6fire_inputs[6] = {
+		"CD In (L)",
+		"CD In (R)",
+		"Line In (L)",
+		"Line In (R)",
+		"Phono In (L)",
+		"Phono In (R)"
+	};
+	static char* dmx6fire_outputs[6] = {
+		"Front (L)",
+		"Front (R)",
+		"Rear (L)",
+		"Rear (R)",
+		"Centre",
+		"LFE"
+	};
+
 
 	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
 	gtk_widget_show(scrolledwindow);
@@ -1240,6 +1501,13 @@ static void create_analog_volume(GtkWidget *main, GtkWidget *notebook, int page)
 		gtk_container_add(GTK_CONTAINER(frame), vbox);
 		gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
 
+		/* Add friendly labels for DMX 6Fires */
+		if((card_eeprom.subvendor == ICE1712_SUBDEVICE_DMX6FIRE) && (i < 6)){
+			label = gtk_label_new(dmx6fire_outputs[i]);
+			gtk_widget_show(label);
+			gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 6);
+		}
+
 		adj = gtk_adjustment_new(0, -(envy_dac_max()), 0, 1, 16, 0);
 		av_dac_volume_adj[i] = adj;
 		vscale = gtk_vscale_new(GTK_ADJUSTMENT(adj));
@@ -1289,6 +1557,13 @@ static void create_analog_volume(GtkWidget *main, GtkWidget *notebook, int page)
 		gtk_container_add(GTK_CONTAINER(frame), vbox);
 		gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
 
+		/* Add friendly labels for DMX 6Fires */
+		if((card_eeprom.subvendor == ICE1712_SUBDEVICE_DMX6FIRE) && (i < 6)){
+			label = gtk_label_new(dmx6fire_inputs[i]);
+			gtk_widget_show(label);
+			gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 6);
+		}
+
 		adj = gtk_adjustment_new(0, -127, 0, 1, 16, 0);
 		av_adc_volume_adj[i] = adj;
 		vscale = gtk_vscale_new(GTK_ADJUSTMENT(adj));
@@ -1338,6 +1613,13 @@ static void create_analog_volume(GtkWidget *main, GtkWidget *notebook, int page)
 		gtk_container_add(GTK_CONTAINER(frame), vbox);
 		gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
 
+		/* Add friendly labels for DMX 6Fires */
+		if((card_eeprom.subvendor == ICE1712_SUBDEVICE_DMX6FIRE) && (i < 6)){
+			label = gtk_label_new(dmx6fire_inputs[i]);
+			gtk_widget_show(label);
+			gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 6);
+		}
+
 		adj = gtk_adjustment_new(0, -36, 0, 1, 16, 0);
 		av_ipga_volume_adj[i] = adj;
 		vscale = gtk_vscale_new(GTK_ADJUSTMENT(adj));
@@ -1360,7 +1642,12 @@ static void create_analog_volume(GtkWidget *main, GtkWidget *notebook, int page)
 
 static void usage(void)
 {
-	fprintf(stderr, "usage: envy24control [-c card#] [-D control-name]\n");
+	fprintf(stderr, "usage: envy24control [-c card#] [-D control-name] [-o num-outputs] [-i num-inputs]\n");
+	fprintf(stderr, "\t-c, --card\tAlsa card number to control\n");
+	fprintf(stderr, "\t-D, --device\tcontrol-name\n");
+	fprintf(stderr, "\t-o, --outputs\tLimit number of outputs to display\n");
+	fprintf(stderr, "\t-i, --input\tLimit number of inputs to display\n");
+	fprintf(stderr, "\t-s, --spdif\tLimit number of spdif outputs to display\n");
 }
 
 int main(int argc, char **argv)
@@ -1373,9 +1660,15 @@ int main(int argc, char **argv)
 	int npfds;
 	struct pollfd *pfds;
 	int page;
+	int output_channels_set = 0;
+	int input_channels_set = 0;
 	static struct option long_options[] = {
 		{"device", 1, 0, 'D'},
 		{"card", 1, 0, 'c'},
+		{"inputs", 1, 0, 'i'},
+		{"outputs", 1, 0, 'o'},
+		{"spdif", 1, 0, 's'},
+		{0, 0, 0, 0}
 	};
 
 
@@ -1386,7 +1679,10 @@ int main(int argc, char **argv)
         gtk_init(&argc, &argv);
 
 	name = "hw:0";
-	while ((c = getopt_long(argc, argv, "D:c:", long_options, NULL)) != -1) {
+	input_channels = 8;
+	output_channels = 10;
+	spdif_channels = 2;
+	while ((c = getopt_long(argc, argv, "D:c:i:o:s:", long_options, NULL)) != -1) {
 		switch (c) {
 		case 'c':
 			i = atoi(optarg);
@@ -1400,6 +1696,29 @@ int main(int argc, char **argv)
 		case 'D':
 			name = optarg;
 			break;
+		case 'i':
+			input_channels = atoi(optarg);
+			if (input_channels < 0 || input_channels > 8) {
+				fprintf(stderr, "envy24control: must have 0-8 inputs\n");
+				exit(1);
+			}
+			input_channels_set = 1;
+			break;
+		case 'o':
+			output_channels = atoi(optarg);
+			if (output_channels < 0 || output_channels > 10) {
+				fprintf(stderr, "envy24control: must have 0-10 outputs\n");
+				exit(1);
+			output_channels_set = 1;
+			}
+			break;
+		case 's':
+			spdif_channels = atoi(optarg);
+			if (spdif_channels < 0 || spdif_channels > 2) {
+				fprintf(stderr, "envy24control: must have 0-2 spdifs\n");
+				exit(1);
+			}
+			break;
 		default:
 			usage();
 			exit(1);
@@ -1428,6 +1747,19 @@ int main(int argc, char **argv)
 	}
 	memcpy(&card_eeprom, snd_ctl_elem_value_get_bytes(val), 32);
 
+	/* Set a better default for input_channels and output_channels */
+	if(!input_channels_set) {
+		if(card_eeprom.subvendor == ICE1712_SUBDEVICE_DMX6FIRE) {
+			input_channels = 6;
+		}
+	}
+
+	if(!output_channels_set) {
+		if(card_eeprom.subvendor == ICE1712_SUBDEVICE_DMX6FIRE) {
+			output_channels = 6;
+		}
+	}
+
 	/* Initialize code */
 	level_meters_init();
 	patchbay_init();
